/*
 ****************************************************************************
 *
 *                   "DHRYSTONE" Benchmark Program
 *                   -----------------------------
 *                                                                            
 *  Version:    C, Version 2.1
 *                                                                            
 *  File:       dhry_1.c (part 2 of 3)
 *
 *  Date:       May 25, 1988
 *
 *  Author:     Reinhold P. Weicker
 *
 *  Modified March 6, 2001 by Laurent Vogel to run under V68k:
 *  - no user input, no floats, 16 ints but long run_index, clock() time
 *    measurement, no malloc(), ANSI C with prototypes.
 *
 ****************************************************************************
 */

#include "dhry.h"

#include "prz.h"
extern int strcmp(char *a, char *b);
extern char *strcpy(char *a, char *b);
extern int strlen(char *a);


/* returns a * b / c */
extern long muldiv(long a, long b, long c);


/* Global Variables: */

Rec_Pointer     Ptr_Glob,
                Next_Ptr_Glob;
int             Int_Glob;
Boolean         Bool_Glob;
char            Ch_1_Glob,
                Ch_2_Glob;
int             Arr_1_Glob [50];
int             Arr_2_Glob [50] [50];


#ifndef REG
        Boolean Reg = false;
#define REG
        /* REG becomes defined as empty */
        /* i.e. no register variables   */
#else
        Boolean Reg = true;
#endif

/* variables for time measurement: */


long            Begin_Time,
                End_Time,
                User_Time,
                Too_Small_Time;
long            Microseconds,
                Dhrystones_Per_Second;
         
/* end of variables for time measurement */

static void 
check_int(char *s, int a, int b)
{
  if(a != b) {
    printf("%s = %d (should be %d)\n", s, a, b);
  }
}

static void 
check_string(char *s, char *a, char *b)
{
  int i,n,c;
  if(strcmp(a,b)) {
    n = strlen(b);
    
    printf("%s = '",s);
    for(i = 0 ; i < n ; i++) {
      c = a[i];
      if(c == 0) break;
      if(c < 32 || c >= 127) {
        printf("\\%c%c%c", '0'+((c>>6)&7), '0'+((c>>3)&7), '0'+(c&7));
      } else {
        putchar(c);
      }
    }
    if(a[i] != 0) {
      printf("...");
    } 
    printf("',\n  (should be '%s')\n", b);
  }
}

void run_dhry(long);

/* 
 * main program, corresponds to procedures        
 * Main and Proc_0 in the Ada version             
 */

int 
main (int argc, char *argv[])
{
  long  Number_Of_Runs, a;
  int ntries = 0;
  Number_Of_Runs = 25000;
  /* wait at least 3 seconds */
  Too_Small_Time = 3 * CLOCKS_PER_SEC;

  printf ("Dhrystone Benchmark, Version 2.1 (Language: C)\n");
  printf ("Adapted to the V68k environment.\n");
  printf ("\n");
  
  printf ("Executing %ld runs through Dhrystone\n", Number_Of_Runs);
  for(;;) {
    run_dhry(Number_Of_Runs);
    if (User_Time < Too_Small_Time) {
      if(ntries == 0) {
        printf ("Measured time too small to obtain meaningful results:\n");
        Number_Of_Runs *= 2;
        printf ("trying a bigger value %ld\n", Number_Of_Runs);
      } else {
        if(Number_Of_Runs * CLOCKS_PER_SEC > 0x3FFFFFFFL) {
          printf ("Either too fast machine, or dead clock. Quitting.\n");
          exit(1);
        }
        Number_Of_Runs *= 2;
        printf ("Still trying a bigger value %ld\n", Number_Of_Runs);
      }
      ntries++;
      
    } else {
      break;
    }
  }
  printf("User_Time = %ld (CLOCKS_PER_SEC = %ld)\n", User_Time, 
    CLOCKS_PER_SEC);
  Microseconds = muldiv(User_Time, 1000000 * 100, 
    Number_Of_Runs*CLOCKS_PER_SEC);
  Dhrystones_Per_Second = muldiv(Number_Of_Runs, CLOCKS_PER_SEC, User_Time);
  printf ("Microseconds for one run through Dhrystone: ");
  printf ("%ld%c%02d \n", Microseconds / 100L, '.', 
      (int)(Microseconds%100L));
  printf ("Dhrystones per Second:                      ");
  printf ("%ld\n", Dhrystones_Per_Second);
  exit(0);
}

Rec_Type Next_Ptr_Glob_Buf;
Rec_Type Ptr_Glob_Buf;
  
void run_dhry(long My_Number_Of_Runs)
{
        One_Fifty       Int_1_Loc;
  REG   One_Fifty       Int_2_Loc;
        One_Fifty       Int_3_Loc;
  REG   char            Ch_Index;
        Enumeration     Enum_Loc;
        Str_30          Str_1_Loc;
        Str_30          Str_2_Loc;
  REG   long            Run_Index;
  REG   long            Number_Of_Runs;

  /* Initializations */
  
  Number_Of_Runs = My_Number_Of_Runs;

#if 0
  Next_Ptr_Glob = (Rec_Pointer) malloc (sizeof (Rec_Type));
  Ptr_Glob = (Rec_Pointer) malloc (sizeof (Rec_Type));
#else
  Next_Ptr_Glob = &Next_Ptr_Glob_Buf;
  Ptr_Glob = &Ptr_Glob_Buf;
#endif

  Ptr_Glob->Ptr_Comp                    = Next_Ptr_Glob;
  Ptr_Glob->Discr                       = Ident_1;
  Ptr_Glob->variant.var_1.Enum_Comp     = Ident_3;
  Ptr_Glob->variant.var_1.Int_Comp      = 40;
  strcpy (Ptr_Glob->variant.var_1.Str_Comp, 
          "DHRYSTONE PROGRAM, SOME STRING");
  strcpy (Str_1_Loc, "DHRYSTONE PROGRAM, 1'ST STRING");

  Arr_2_Glob [8][7] = 10;
        /* Was missing in published program. Without this statement,    */
        /* Arr_2_Glob [8][7] would have an undefined value.             */
        /* Warning: With 16-Bit processors and Number_Of_Runs > 32000,  */
        /* overflow may occur for this array element.                   */


  /***************/
  /* Start timer */
  /***************/
 
  Begin_Time = clock();
  
  for (Run_Index = 1; Run_Index <= Number_Of_Runs; ++Run_Index)
  {

    Proc_5();
    Proc_4();
      /* Ch_1_Glob == 'A', Ch_2_Glob == 'B', Bool_Glob == true */
    Int_1_Loc = 2;
    Int_2_Loc = 3;
    strcpy (Str_2_Loc, "DHRYSTONE PROGRAM, 2'ND STRING");
    Enum_Loc = Ident_2;
    Bool_Glob = ! Func_2 (Str_1_Loc, Str_2_Loc);
      /* Bool_Glob == 1 */
    while (Int_1_Loc < Int_2_Loc)  /* loop body executed once */
    {
      Int_3_Loc = 5 * Int_1_Loc - Int_2_Loc;
        /* Int_3_Loc == 7 */
      Proc_7 (Int_1_Loc, Int_2_Loc, &Int_3_Loc);
        /* Int_3_Loc == 7 */
      Int_1_Loc += 1;
    } /* while */
      /* Int_1_Loc == 3, Int_2_Loc == 3, Int_3_Loc == 7 */
    Proc_8 (Arr_1_Glob, Arr_2_Glob, Int_1_Loc, Int_3_Loc);
      /* Int_Glob == 5 */
    Proc_1 (Ptr_Glob);
    for (Ch_Index = 'A'; Ch_Index <= Ch_2_Glob; ++Ch_Index)
                             /* loop body executed twice */
    {
      if (Enum_Loc == Func_1 (Ch_Index, 'C'))
          /* then, not executed */
        {
        Proc_6 (Ident_1, &Enum_Loc);
        strcpy (Str_2_Loc, "DHRYSTONE PROGRAM, 3'RD STRING");
        Int_2_Loc = (int) Run_Index;
        Int_Glob = (int) Run_Index;
        }
    }
      /* Int_1_Loc == 3, Int_2_Loc == 3, Int_3_Loc == 7 */
    Int_2_Loc = Int_2_Loc * Int_1_Loc;
    Int_1_Loc = Int_2_Loc / Int_3_Loc;
    Int_2_Loc = 7 * (Int_2_Loc - Int_3_Loc) - Int_1_Loc;
      /* Int_1_Loc == 1, Int_2_Loc == 13, Int_3_Loc == 7 */
    Proc_2 (&Int_1_Loc);
      /* Int_1_Loc == 5 */

  } /* loop "for Run_Index" */

  /**************/
  /* Stop timer */
  /**************/

  End_Time = clock();
  
  User_Time = End_Time - Begin_Time;

  if (User_Time < Too_Small_Time) {
    return;
  }

  printf ("Execution ends. Checking variable values ...\n");
  
#if 0
  printf ("Int_Glob:            %d\n", Int_Glob);
  printf ("        should be:   %d\n", 5);
  printf ("Bool_Glob:           %d\n", Bool_Glob);
  printf ("        should be:   %d\n", 1);
  printf ("Ch_1_Glob:           %c\n", Ch_1_Glob);
  printf ("        should be:   %c\n", 'A');
  printf ("Ch_2_Glob:           %c\n", Ch_2_Glob);
  printf ("        should be:   %c\n", 'B');
  printf ("Arr_1_Glob[8]:       %d\n", Arr_1_Glob[8]);
  printf ("        should be:   %d\n", 7);
  printf ("Arr_2_Glob[8][7]:    %d\n", Arr_2_Glob[8][7]);
  printf ("        should be:   Number_Of_Runs + 10\n");
#else
  check_int("Int_Glob", Int_Glob, 5);
  check_int("Bool_Glob", Bool_Glob, 1);
  check_int("Ch_1_Glob", Ch_1_Glob, 'A');
  check_int("Ch_2_Glob", Ch_2_Glob, 'B');
  check_int("Arr_1_Glob[8]", Arr_1_Glob[8], 7);
  check_int("Arr_2_Glob[8][7]", Arr_2_Glob[8][7], (int)Number_Of_Runs + 10);
#endif
  
#if 0
  printf ("Ptr_Glob->\n");
  printf ("  Ptr_Comp:          %d\n", (int) Ptr_Glob->Ptr_Comp);
  printf ("        should be:   (implementation-dependent)\n");
  printf ("  Discr:             %d\n", Ptr_Glob->Discr);
  printf ("        should be:   %d\n", 0);
  printf ("  Enum_Comp:         %d\n", Ptr_Glob->variant.var_1.Enum_Comp);
  printf ("        should be:   %d\n", 2);
  printf ("  Int_Comp:          %d\n", Ptr_Glob->variant.var_1.Int_Comp);
  printf ("        should be:   %d\n", 17);
  printf ("  Str_Comp:          %s\n", Ptr_Glob->variant.var_1.Str_Comp);
  printf ("        should be:   DHRYSTONE PROGRAM, SOME STRING\n");
#else
  check_int("Ptr_Glob->Discr", Ptr_Glob->Discr, 0);
  check_int("Ptr_Glob->Enum_Comp", Ptr_Glob->variant.var_1.Enum_Comp, 2);
  check_int("Ptr_Glob->Int_Comp", Ptr_Glob->variant.var_1.Int_Comp, 17);
  check_string("Ptr_Glob->Str_Comp", Ptr_Glob->variant.var_1.Str_Comp, 
    "DHRYSTONE PROGRAM, SOME STRING");
#endif
    
#if 0
  printf ("Next_Ptr_Glob->\n");
  printf ("  Ptr_Comp:          %d\n", (int) Next_Ptr_Glob->Ptr_Comp);
  printf ("        should be:   (implementation-dependent), same as above\n");
  printf ("  Discr:             %d\n", Next_Ptr_Glob->Discr);
  printf ("        should be:   %d\n", 0);
  printf ("  Enum_Comp:         %d\n", Next_Ptr_Glob->variant.var_1.Enum_Comp);
  printf ("        should be:   %d\n", 1);
  printf ("  Int_Comp:          %d\n", Next_Ptr_Glob->variant.var_1.Int_Comp);
  printf ("        should be:   %d\n", 18);
  printf ("  Str_Comp:          %s\n",
                                Next_Ptr_Glob->variant.var_1.Str_Comp);
  printf ("        should be:   DHRYSTONE PROGRAM, SOME STRING\n");
#else
  check_int("Next_Ptr_Glob->Discr", Next_Ptr_Glob->Discr, 0);
  check_int("Next_Ptr_Glob->Enum_Comp", 
    Next_Ptr_Glob->variant.var_1.Enum_Comp, 1);
  check_int("Next_Ptr_Glob->Int_Comp", 
    Next_Ptr_Glob->variant.var_1.Int_Comp, 18);
  check_string("Next_Ptr_Glob->Str_Comp", 
    Next_Ptr_Glob->variant.var_1.Str_Comp, 
    "DHRYSTONE PROGRAM, SOME STRING");
#endif

#if 0
  printf ("Int_1_Loc:           %d\n", Int_1_Loc);
  printf ("        should be:   %d\n", 5);
  printf ("Int_2_Loc:           %d\n", Int_2_Loc);
  printf ("        should be:   %d\n", 13);
  printf ("Int_3_Loc:           %d\n", Int_3_Loc);
  printf ("        should be:   %d\n", 7);
  printf ("Enum_Loc:            %d\n", Enum_Loc);
  printf ("        should be:   %d\n", 1);
  printf ("Str_1_Loc:           %s\n", Str_1_Loc);
  printf ("        should be:   DHRYSTONE PROGRAM, 1'ST STRING\n");
  printf ("Str_2_Loc:           %s\n", Str_2_Loc);
  printf ("        should be:   DHRYSTONE PROGRAM, 2'ND STRING\n");
#else
  check_int("Int_1_Loc", Int_1_Loc, 5);
  check_int("Int_2_Loc", Int_2_Loc, 13);
  check_int("Int_3_Loc", Int_3_Loc, 7);
  check_int("Enum_Loc", Enum_Loc, 1);
  check_string("Str_1_Loc", Str_1_Loc, 
    "DHRYSTONE PROGRAM, 1'ST STRING");
  check_string("Str_2_Loc", Str_2_Loc, 
    "DHRYSTONE PROGRAM, 2'ND STRING");
#endif
    
  printf ("Check done.\n");
}


void
Proc_1 (REG Rec_Pointer Ptr_Val_Par)
/******************/
    /* executed once */
{
  REG Rec_Pointer Next_Record = Ptr_Val_Par->Ptr_Comp;  
                                        /* == Ptr_Glob_Next */
  /* Local variable, initialized with Ptr_Val_Par->Ptr_Comp,    */
  /* corresponds to "rename" in Ada, "with" in Pascal           */
  
  structassign (*Ptr_Val_Par->Ptr_Comp, *Ptr_Glob); 
  Ptr_Val_Par->variant.var_1.Int_Comp = 5;
  Next_Record->variant.var_1.Int_Comp 
        = Ptr_Val_Par->variant.var_1.Int_Comp;
  Next_Record->Ptr_Comp = Ptr_Val_Par->Ptr_Comp;
  Proc_3 (&Next_Record->Ptr_Comp);
    /* Ptr_Val_Par->Ptr_Comp->Ptr_Comp 
                        == Ptr_Glob->Ptr_Comp */
  if (Next_Record->Discr == Ident_1)
    /* then, executed */
  {
    Next_Record->variant.var_1.Int_Comp = 6;
    Proc_6 (Ptr_Val_Par->variant.var_1.Enum_Comp, 
           &Next_Record->variant.var_1.Enum_Comp);
    Next_Record->Ptr_Comp = Ptr_Glob->Ptr_Comp;
    Proc_7 (Next_Record->variant.var_1.Int_Comp, 10, 
           &Next_Record->variant.var_1.Int_Comp);
  }
  else /* not executed */
    structassign (*Ptr_Val_Par, *Ptr_Val_Par->Ptr_Comp);
} /* Proc_1 */


void
Proc_2 (One_Fifty * Int_Par_Ref)
/******************/
    /* executed once */
    /* *Int_Par_Ref == 1, becomes 4 */

{
  One_Fifty  Int_Loc;  
  Enumeration   Enum_Loc;

  Int_Loc = *Int_Par_Ref + 10;
  do /* executed once */
    if (Ch_1_Glob == 'A')
      /* then, executed */
    {
      Int_Loc -= 1;
      *Int_Par_Ref = Int_Loc - Int_Glob;
      Enum_Loc = Ident_1;
    } /* if */
  while (Enum_Loc != Ident_1); /* true */
} /* Proc_2 */


void
Proc_3 (Rec_Pointer *Ptr_Ref_Par)
/******************/
    /* executed once */
    /* Ptr_Ref_Par becomes Ptr_Glob */

{
  if (Ptr_Glob != Null)
    /* then, executed */
    *Ptr_Ref_Par = Ptr_Glob->Ptr_Comp;
  Proc_7 (10, Int_Glob, &Ptr_Glob->variant.var_1.Int_Comp);
} /* Proc_3 */


void
Proc_4 (void)
/*******/
    /* executed once */
{
  Boolean Bool_Loc;

  Bool_Loc = Ch_1_Glob == 'A';
  Bool_Glob = Bool_Loc | Bool_Glob;
  Ch_2_Glob = 'B';
} /* Proc_4 */

void
Proc_5 (void) 
/*******/
    /* executed once */
{
  Ch_1_Glob = 'A';
  Bool_Glob = false;
} /* Proc_5 */


        /* Procedure for the assignment of structures,          */
        /* if the C compiler doesn't support this feature       */
#ifdef  NOSTRUCTASSIGN
memcpy (d, s, l)
register char   *d;
register char   *s;
register int    l;
{
        while (l--) *d++ = *s++;
}
#endif


