/*
  mkgenhdr.c : patch the genesis header from the mxrom result.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define VERSION "version 0.0, 08 march 2001"

typedef unsigned char uchar;
typedef unsigned int uint;

#define CKSUM_OFF 0x18E
#define ROMEND_OFF 0x1A4

uchar buf[0x4000], hdrbuf[0x200];

void set_long(uchar *uaddr, long value)
{
  uaddr[0] = value >> 24;
  uaddr[1] = value >> 16;
  uaddr[2] = value >> 8;
  uaddr[3] = value;
}

short get_word(uchar *uaddr)
{
  return (uaddr[0] << 8) + uaddr[1];
}

void set_word(uchar *uaddr, short value)
{
  uaddr[0] = value >> 8;
  uaddr[1] = value;
}

/* return codes :
  0: success
  1: file io error
  2: short read
  3: write error
  4: misc error
*/
int do_the_job(char *srcname, char *destname)
{
  size_t len, count;
  int sum;
  int i;
  long romlength, addlength;
  FILE *from, *to;
  
  from = fopen(srcname, "rb");
  if(from == NULL) return 1;
  to = fopen(destname, "wb");
  if(to == NULL) return 1;
  len = 0x200;
  count = fread(hdrbuf, 1, len, from);
  if(count != len) return 2;
  
  /* from now on, simply read the rest of the rom, 
  ** to compute the checksum 
  */
  romlength = 0x200;
  sum = 0;
  do {
    len = 0x4000;
    count = fread(buf, 1, len, from);
    len = count;
    romlength += len;
      
    for(i = 0 ; i < len ; i += 2) {
      sum += get_word(buf+i);
    }
    if(len & 1) {
      buf[len+1] = 0;
      sum += get_word(buf+len-1);
    }
  } while(len > 0);

  if(romlength < 0x220) return 2;
  addlength = ((romlength + 0x3FFF) & (~0x3FFF)) - romlength;
  
  /* now, I ve got all info needed to write the header : */

  set_word(hdrbuf + CKSUM_OFF, sum);
  set_long(hdrbuf + ROMEND_OFF, romlength + addlength);
  /* ... */
  
  len = 0x200;
  count = fwrite(hdrbuf, 1, len, to);
  if(count != len) return 2;
  
  i = fseek(from, 0x200L, SEEK_SET);
  if(i != 0) {
    fprintf(stderr, "could not seek"); 
    return 1;
  }
  do {
    len = 0x4000;
    count = fread(buf, 1, len, from);
    len = count;
    count = fwrite(buf, 1, len, to);
    if( len != count) return 2;
  } while(len > 0);

  /* pad ROM size to 16 Kbytes */
  if(addlength) {
    for(i = 0 ; i < addlength ; i++){
      buf[i] = 0;
    }
    len = addlength;
    count = fwrite(buf, 1, len, to);
    if( len != count) return 2;
  }

  fclose(from);
  fclose(to);
  return 0;
}

int main(int argc, char *argv[])
{
  char *src_name ;
  char *dest_name ;
  int err;

  while(argc >= 1) {
    if(!strcmp("--help", argv[1])) {
      /* TODO */
      goto usage;
    } else if(!strcmp("--version", argv[1])) {
      printf(VERSION);
      printf("\n");
      exit(0);
    } else {
      break;
    }
  }

  if(argc == 1) {
    src_name = "rom.img" ;
    dest_name = "rom.bin" ;
  } else if(argc == 2) {
    src_name = argv[1] ;
    dest_name = "rom.bin" ;
  } else if(argc == 3) {
    src_name = argv[1] ;
    dest_name = argv[2] ;
  } else {
    goto usage;
  }

  err = do_the_job(src_name, dest_name) ;
  switch(err) {
  case 0:
    exit(0);
  default:
    fprintf(stderr, "error number %d\n", err);
    exit(1);
  }
  
usage:
  fprintf(stderr, "usage: %s [<src> [<dest>]]\n", argv[0]);
  fprintf(stderr, 
"<src> and <dest> default to rom.img and rom.bin respectively\n");
  exit(1);
}
