dnl
dnl 
dnl start.m4 : the start of ROM, and the exception vectors.
dnl
dnl

dnl
dnl macro forloop taken verbatim from GNU m4 man page.
dnl

define(`forloop',
        `pushdef(`$1', `$2')_forloop(`$1', `$2', `$3', `$4')popdef(`$1')')

define(`_forloop',
       `$4'`ifelse($1, `$3', ,
                   `define(`$1', incr($1))_forloop(`$1', `$2', `$3', `$4')')')

define(`repeat',`forloop(`i',1,`$1',`$2')')

!
! memory map :
! 0 - ... ROM 
! FF0000 : start of USER RAM
! FF0000 + sizeof(struct globals) : start of free mem
! FFE000 : top of user RAM.
! FFF000 : top of user stack.
! FFFF00 : top of super stack, start of globals
! .....0 : long tick_count (incremented every vbl)
! ...      (other globals, dealt with by C part)
! FFFFFF : last bit of RAM.
!


    .sect text
    .sect rom
    .sect data
    .sect bss


	.sect text
	.def`'ine _start
romstart:
_start: 
	.data4 0xFFFF00	! reset SSP 
	.data4 begin	  ! reset PC
  .data4 bus      ! bus error 
  .data4 address  ! address error

forloop(`i', 4, 29, `dnl
	.data4 0x200 + eval(4*i)
')
  ! vector 30: level 6 autovectored interrupt (VBL)
  .data4 tick
  
forloop(`i', 31, 32, `dnl
	.data4 0x200 + eval(4*i)
')

  ! vector 33: trap #1
  .data4 dotrap1
 
forloop(`i', 34, 63, `dnl bsr exception, in word format
	.data4 0x200 + eval(4*i)
')


.assert . == romstart + 0x100

! room for the ROM header
	.ascii "SEGA MEGA DRIVE "
	.ascii "(C)SEGA 1988.JUL"
	.ascii "VRUN GENESIS    "
	.ascii "                "
	.ascii "                "
	.ascii "VRUN GENESIS    "
	.ascii "                "
	.ascii "                "
  .ascii "GM 20010308-00"
  .data2 0  ! checksum at address 0x18E
  .ascii "J               "
  .data4 0  ! ROM start
  .data4 0  ! ROM end at address 0x1A4
  .data4 0x00FF0000, 0x00FFFFFF  ! RAM start and end
  .ascii "                "
  .ascii "                "
  .ascii "                "
  .ascii "                "
  .ascii "JUE             "
  
.assert . == romstart + 0x200
	
  nop
  nop
  nop
  nop
br_table:
forloop(`i', 2, 63, `dnl bsr exception, in word format
	.data2 0x6100
	.data2 exception - .
')

.assert . == romstart + 0x300

address:
bus:
  move.l 2(a7),a0
  move.l 10(a7),a1
  lea 0xFFFF00,a7
	move.l a1,-(a7)
	move.l a0,-(a7)
	pea bus_msg
	jsr _printf
  bra end

exception:
	move.l (a7)+,d0
	move.l (a7)+,a1
	lea br_table,a0
	sub.l a0,d0
	lsr.w #2,d0
	add.l #2,d0
	move.l d0,d7
doexcept:
	lea 0xFFFF00,a7
	move.l a1,-(a7)
	move.w d7,-(a7)
	pea except_msg
	jsr _printf
end:
	bra end

except_msg:
	.ascii "fatal: exception %d at pc 0x%08lx"
	.data1 10,0
bus_msg:
  .ascii "bus or address error, pc = %08lx, addr = %08lx"
  .data1 10,0
welcome_msg:
  .ascii "Hello."
  .data1 10,0
	.align 2

begin:
  ! no interrupts
	move.w #0x2700,sr

	! setup super and user stack
	lea 0xFFFF00,a7
	lea 0xFFF000,a0
	move.l a0,usp

	! Sega Security Code
	move.b  0xA10001,d0             ! Get version number
  and.b  #0x0F,d0                
  beq 1f                          ! If not version $0
  move.l  #0x53454741,0xA14000    ! Output ASCII 'SEGA'
1:
	
  ! init the video and the clock.
  jsr _init_video

  ! print welcome message
	pea welcome_msg
	jsr _printf
	add.l #4,a7

	! allow interrupts. 
	move.w #0x2000,sr
	
	! okay, enough assembler, go in C part
	jsr _vrun
	move.w #0,-(sp)
  bsr _exit
  bra end

  ! _putchar uses trap because it needs to write to
  ! video registers directly (hence needs supervisor mode).
  .extern _putchar
_putchar:
  jmp _do_putchar
!  trap #1
!  rts

  ! called at each VBL interrupt
tick:
  add.l #1,0xFFFF00
  rte
  
  ! trap 1 handler, trap #1 means _putchar.
dotrap1:
  move.w (sp),d0
  ! use super stack, 
  lea 6(a7),a0
  btst #13,d0
  bne 1f
  ! was in user mode, so use user stack :
  move.l usp,a0
1:
  move.w (a0),d0
  move.w d0,-(sp)
  jsr _do_putchar
  add.l #2,sp
  rte

	.extern _usermode
_usermode:
	move.l 4(sp),a0
	move.l 8(sp),a1
	and.w #0x0700,sr
	move.l a1,-(sp)
	pea userexit(pc)
	jmp (a0)

userexit:
	move.w #0,-(sp)
	bsr _exit

	.extern _exit
_exit:
	move.w 4(sp),d0
	pea exit_msg
	jsr _printf
	bra end

exit_msg:
  .ascii "exit(%d)"
  .data1 10,0
  .align 2


