/*
  video.c
*/

extern void set_vreg(int reg_num, int value);
extern int get_vstat(void);
#define BANK_VRAM 1
#define BANK_CRAM 3
#define BANK_VSRAM 5
extern void set_vaddr(int bank, unsigned addr); 
/* use set_vaddr to select the bank, then use set/copy/fill_vram
** to effectively put data.
*/
extern void set_vram(int value);
extern void copy_vram(int *from, unsigned size);
extern void fill_vram(int value, unsigned size);

#define CHAR_MIN 32
#define CHAR_MAX 128
#define NUM_CHARS (CHAR_MAX - CHAR_MIN)

extern long the_font[];

struct globals {
  long ticks;
  int row;
  int col;
  long clocks_per_sec;
} *g = (struct globals *) 0xFFFF00;

void *get_ram_start(void)
{
  return (void *)0xFF0000;
}

long get_ram_size(void)
{
  return 0xE000;
}

void init_video(void)
{
 
  /* initialize video regs */
  set_vreg( 0, 0x04);  /* no H int, enable read HV counter */
  set_vreg( 1, 0x64);  /* enable display, V int, no DMA, 28 LINES (PAL) */
#define LINES 28
  set_vreg( 2, 0x08);  /* Scroll A addr is 0x2000 */
#define MAP_ADDR 0x2000
  set_vreg( 3, 0x3C);  /* WINDOW addr is 0xF000 */
  set_vreg( 4, 0x07);  /* Scroll B is 0xE000 */
  set_vreg( 5, 0x60);  /* sprite addr is 0xD000 */
  set_vreg( 6, 0);
  set_vreg( 7, 0x00);  /* background color is color 0 of palette 0 */
  set_vreg( 8, 0);
  set_vreg( 9, 0);
  set_vreg(10, 0);    /* number of H iterrupts per line */
  set_vreg(11, 0);    /* no external int, full V and H scroll */
  set_vreg(12, 0x81); /* 40 cells wide, no shadow, no interlace */
#define COLS 40
  set_vreg(13, 0x30); /* H scroll addr is 0xC000 */
  set_vreg(14, 0);
  set_vreg(15, 2);    /* 2 bytes added after each RAM access */
  set_vreg(16, 0x01); /* scrolls are 64 COLS per 32 lines */
#define DELTA 64
  set_vreg(17, 0);    /* window H position */
  set_vreg(18, 0);    /* window V position */
  set_vreg(19, 0);    /* DMA counter low = 0 */
  set_vreg(20, 0);    /* DMA counter high = 0*/
  set_vreg(21, 0);    /* DMA addr low = 0 */
  set_vreg(22, 0);    /* DMA addr mid = 0 */
  set_vreg(23, 0x80); /* DMA addr high = 0, mode VRAM Fill */
 
  /* init the palette */
  set_vaddr(BANK_CRAM, 0);
  set_vram(0x0000);  /* palette 0, color 0 is black */
  set_vram(0x0eee);  /* palette 0, color 1 is white */
  fill_vram(0, 62);  /* 1 word per color, 16 color per palette */
    /* and 4 palettes */
    
  /* init the scroll ram */
  set_vaddr(BANK_VSRAM, 0);
  fill_vram(0, 64);  /* ??? */ 
  
  /* clear the screen */
  set_vaddr(BANK_VRAM, MAP_ADDR);
  fill_vram(0, DELTA * LINES);
  
  /* clear the misc VRAM area */
  set_vaddr(BANK_VRAM, 0xC000);
  fill_vram(0, 0x2000);
  
  /* load the font */
  set_vaddr(BANK_VRAM, 0);
  copy_vram((int *)the_font, NUM_CHARS * 16);
  
  g->row = 0;
  g->col = 0;
  
  /* init the clock */
  
  g->ticks = 0;
  if(get_vstat() & 1) {
    /* PAL mode */
    g->clocks_per_sec = 50;
  } else {
    /* NTSC mode */
    g->clocks_per_sec = 60;
  }

}

long clocks_per_sec(void)
{
  return g->clocks_per_sec;
}
  
long clock(void)
{
  return g->ticks;
}

void do_putchar(int c)
{
  if(c == 10) {
    g->row++;
    g->col = 0;
    if(g->row >= LINES) {
      /* wrap up */
      g->row = 0;
    }
    /* clear the line */
    set_vaddr(BANK_VRAM, MAP_ADDR + 2 * DELTA * g->row);
    fill_vram(0, COLS);
    return;
  }
  if(c < CHAR_MIN || c >= CHAR_MAX) return;
  if(g->col >= COLS) {
    g->row++;
    g->col = 0;
    if(g->row >= LINES) {
      /* wrap up */
      g->row = 0;
    }
    /* clear the line */
    set_vaddr(BANK_VRAM, MAP_ADDR + 2 * DELTA * g->row);
    fill_vram(0, COLS);
  }
  if(g->row >= LINES) {
    g->row = 0;
  }
  set_vaddr(BANK_VRAM, MAP_ADDR + 2 * (g->col + DELTA * g->row));
  set_vram(c - CHAR_MIN);
  g->col++;
}

