
#include "tos.h"

extern int printf(char *, ...);

/* taken from mint/cookie.h */
union clong {
        char    aschar[4];
        long    aslong;
};

struct cookie {
        union clong tag;
        long value;
};

typedef struct cookie COOKIE;

#define CJAR    ((COOKIE **) 0x5a0L)
/* end of mcmnt95/cookie.h */

/* taken from mcmnt95/locore.h */
struct sysvars {
        long version;
        void (*vbl)(void);
        long vbls;
        long vblq;
        void (*timer)(void);
        long hz200;
        long ticks;
        void (*five)(void);
        long sysbase;
        long logbase;
        void *saveda5;
        void *savedsp;
        void (*put_vector)(void);
        void (*pull_vector)(void);
        void (*eventloop)(void);
        long (*rwabs)(short, void *, short, short, short, long);
        long (*mediach)(short);
        long (*getbpb)(short);
        long drvmap;
        long term;
        long crit;
        long resvec;
        long resval;
        struct { long buf[20]; } procinfo;
};
/* end of mcmnt95/locore.h */

struct sysvars *sysvar = 0;

static int same_cookie(char *a, char *b)
{
  return (a[0] == b[0]) && (a[1] == b[1]) && 
    (a[2] == b[2]) && (a[3] == b[3]);
}

void init_sysvar(void)
{
  COOKIE *jar;
  sysvar = 0;
  
  jar = *CJAR;    /* CJAR defined in cookie.h */
  if (jar) {
    while (jar->tag.aslong != 0) {
      if (same_cookie(jar->tag.aschar, "SVAR")) {
	sysvar = (struct sysvars *)jar->value;
      }
      jar++;
    }
  }
  if(sysvar == 0) {
    printf("cannot initialize sysvar\n");
    Pterm(1);
  }
}

long clocks_per_sec(void)
{
  return 200;
}

long clock(void)
{
  return sysvar->hz200;
}

void putchar(int c)
{
  Bconout(3,c);
}

struct funcs {
  long (*clocks_per_sec)(void);
  long (*clock)(void);
  void (*putchar)(int);
  int (*printf)(char *, ...);
};

struct funcs f = {
  clocks_per_sec, 
  clock, 
  putchar,
  printf
};

/* from basepage.h */
typedef struct basep {
  long            p_lowtpa;       /* pointer to self (bottom of TPA) */
  long            p_hitpa;        /* pointer to top of TPA + 1 */
  long            p_tbase;        /* base of text segment */
  long            p_tlen;         /* length of text segment */
  long            p_dbase;        /* base of data segment */
  long            p_dlen;         /* length of data segment */
  long            p_bbase;        /* base of BSS segment */
  long            p_blen;         /* length of BSS segment */
  char            *p_dta;         /* pointer to current DTA */
  struct basep    *p_parent;      /* pointer to parent's basepage */
  long            p_flags;        /* memory usage flags */
  char            *p_env;         /* pointer to environment string */
/* Anything after this (except for p_cmdlin) is undocumented, reserved,
 * subject to change, etc. -- user programs must NOT use!
 */
  char            p_devx[6];      /* real handles of the standard devices */
  char            p_res2;         /* reserved */
  char            p_defdrv;       /* default drv */
  long            p_undef[17];    /* reserved space */
  long            p_usp;          /* a fake USP to make dLibs programs happy */
  char            p_cmdlin[128];  /* command line image */
} BASEPAGE;
/* end of basepage.h */

int main()
{
  char *fname = "thing.prz";
  BASEPAGE *basepage;
  void (*go)(struct funcs *);

  init_sysvar();

  printf("running \"%s\".\n", fname);

  basepage = (BASEPAGE *) Pexec(3,fname,0L,0L);
  if(basepage == 0) {
    printf("could not Pexec()\n");
    Pterm(1);
  }
  go = (void (*)()) (basepage->p_tbase);
  go(&f);
  printf("Done\n");
  Pterm(0);
}








