/*
  MINT version of the V68k runner.
  compile with : gcc -mshort vrunmint.c printf.c -o vrun
  Was tested on KGMD stock gcc compiler.
*/

#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/times.h>
#include <time.h>

long clocks_per_sec(void)
{
  return CLOCKS_PER_SEC;
}

long my_clock(void)
{
  return clock();
}

struct funcs {
  int version;
  long (*clocks_per_sec)(void);
  long (*clock)(void);
  void (*my_putchar)(int);
  int (*my_printf)(char *, ...);
};

void my_putchar(int c)
{
  if(c == 10) {
    fprintf(stderr, "\n");
  } else {
    fprintf(stderr, "%c", c);
  }
}

/* I don't use MiNT printf because in fact I need an unbuffered printf.*/
#ifdef printf
#undef printf
#endif
#define printf my_printf

extern int printf(char *fmt, ...);

struct funcs f = {
  0x0100,
  clocks_per_sec, 
  my_clock, 
  my_putchar,
  printf
};

struct tos_hdr {
  unsigned char prg_magic[2] ;
#define PRG_MAGIC 0x601a
  unsigned char prg_tsize[4] ;
  unsigned char prg_dsize[4] ;
  unsigned char prg_bsize[4] ;
  unsigned char prg_ssize[4] ;
  unsigned char prg_res1[4] ;    /* reserved, 0 */
  unsigned char prg_flags[4] ;   /* 0 for private memory */
  unsigned char prg_abs[2] ;     /* 0 if reloc info */
};

#if 0
#define GET_LONG(addr) (( (unsigned long)((unsigned char *) (addr))[0] << 24 ) | \
			( ((unsigned char *) (addr))[1] << 16 ) | \
			( ((unsigned char *) (addr))[2] << 8 ) | \
			( ((unsigned char *) (addr))[3] ))
#else
#define GET_LONG(addr) (*(unsigned long *)addr)
#endif

/* will load and relocate and return address where it is loaded */
char * pexec(char *fname)
{
  size_t count, len;
  long rsize, tsize, ssize, bsize, dsize;
  long total;
  unsigned char *addr;
  unsigned char *reloc;
  unsigned char *a, *r;
  FILE *f;
  struct tos_hdr hdr;
  
  f = fopen(fname, "rb");
  if(f == NULL) {
    fprintf(stderr, "pexec: cannot open %s\n", fname);
    return NULL;
  }
  len = sizeof(hdr);
  count = fread(&hdr, 1, len, f);
  if(count != len) {
    fprintf(stderr, "pexec: cannot read header\n");
    fclose(f);
    return NULL;
  }
  tsize = GET_LONG(&hdr.prg_tsize);
  dsize = GET_LONG(&hdr.prg_dsize);
  bsize = GET_LONG(&hdr.prg_bsize);
  ssize = GET_LONG(&hdr.prg_ssize);
 
  total = tsize + dsize + bsize;
  if(total > 0x7FFF) {
    fprintf(stderr, "pexec : file too big for malloc\n");
    fclose(f);
    return NULL;
  }
  addr = malloc((size_t)total);
  if(addr == NULL) {
    fprintf(stderr, "pexec : not enough memory\n");
    fclose(f);
    return NULL;
  }
  /* load in memory */
  len = tsize + dsize;
  count = fread(addr, 1, len, f);
  if(count != len) {
    fprintf(stderr, "pexec: short read\n");
    fclose(f);
    return NULL;
  }
  /* clear bss */
  for(a = addr + tsize + dsize; a < addr + total; *a++ = 0)
    ;
  
  /* skip symb table */
  if(ssize > 0) {
    fseek(f, ssize, SEEK_CUR);
  }
  /* load reloc info */
  rsize = ftell(f);
  fseek(f, 0L, SEEK_END);
  rsize = ftell(f) - rsize;
  fseek(f, -rsize, SEEK_END);
  reloc = malloc((size_t)rsize);
  
  if(reloc == NULL) {
    fprintf(stderr, "pexec: not enough memory\n");
    fclose(f);
    return NULL;
  }
  len = rsize;
  count = fread(reloc, 1, len, f);
  if(count != len) {
    fprintf(stderr, "pexec: short read (relocs)\n");
    fclose(f);
    return NULL;
  }
  fclose(f);

#if 0
  for(r = reloc+4 ; r < reloc+rsize ; r++) {
    fprintf(stderr, "%d, ", *r);
  }
  fprintf(stderr, "\n");
#endif
  
  /* relocate */
  a = addr + GET_LONG(reloc);
  if(a >= addr+total) {
    fprintf(stderr, "pexec: reloc out of bounds %d\n", 1);
    return NULL;
  }
  if(a == addr) {
    /* no relocs */
    free(reloc);
    return addr;
  }
  (*(long *)a) += (long)addr;
  
  r = reloc + 4;
  while(*r) {
    if(r >= reloc+rsize) {
      fprintf(stderr, "pexec: reloc out of bounds %d\n", 2);
      return NULL;
    }
    if(*r == 1) {
      a += 254;
      if(a >= addr+total) {
        fprintf(stderr, "pexec: reloc out of bounds %d\n", 3);
        return NULL;
      }
    } else if(*r & 1) {
      fprintf(stderr, "pexec: illegal reloc *r = %d\n", *r);
      return NULL;
    } else {
      a += *r;
      if(a >= addr+total) {
        fprintf(stderr, "pexec: reloc out of bounds %d\n", 4);
        return NULL;
      }
      (*(long *)a) += (long)addr;
    }
    r++;
  }
  free(reloc);    
  /* that_s it! */
  printf("reloc done: addr=%08lx\n", (long)addr);
  
  return addr;
}
  

int main(int argc, char *argv[])
{
  char *fname = "thing.prz";
  void (*go)(struct funcs *);

  if(argc >= 2) {
    fname = argv[1];
  } 
  printf("running \"%s\".\n", fname);
  
  go = (void (*)()) pexec(fname);
  
  if(go == NULL) {
    printf("could not pexec()\n");
    exit(1);
  }
  go(&f);
  printf("done\n");
  exit(0);
}

  
